<?php
/*
Plugin Name: SEO Blog by SMMIX.tools
Plugin URI: https://smmix.tools/docs/seo-blog#wp
Description: Automatically publish SEO-optimized blog posts from SMMIX.tools to your WordPress site.
Version: 1.7.2
Author: SMMIX.tools
Author URI: https://smmix.tools/services/seo-blog
License: GPL v2 or later
*/

if (!defined('ABSPATH')) exit;

class SMMIX_Blog_Integration
{
    private $plugin_options = 'smmix_blog_options';

    private $webhook_endpoint = 'smmix-webhook';

    private $page_slug = 'smmix-blog';

    // Detect lang & seo plugins
    private $seo_plugins = [
      'yoast' => 'WPSEO_VERSION',
      'rank_math' => 'RANK_MATH_VERSION',
      'all_in_one' => 'AIOSEOP_VERSION'
    ];

    private $language_plugins = [
      'wp_multilang' => 'WPM_PLUGIN_FILE',
      'polylang' => 'POLYLANG_VERSION',
      'wpml' => 'ICL_SITEPRESS_VERSION'
    ];

    public function __construct()
    {
        register_activation_hook(__FILE__, [$this, 'activate_plugin']);

        add_action('admin_menu', [$this, 'add_admin_menu']);
        add_action('admin_init', [$this, 'register_settings']);
        add_action('add_meta_boxes', [$this, 'register_smmix_seoblog_params']);
        add_action('init', [$this, 'register_webhook_endpoint']);

        add_filter("pre_update_option_{$this->plugin_options}", [$this, 'preserve_token'], 10, 2);
        add_filter('plugin_action_links_' . plugin_basename(__FILE__), [$this, 'add_settings_link']);
    }

    private function log_error($message, $type = 'error')
    {
        if ($type !== 'error' && $type !== 'warning') return;
        $log_file = __DIR__ . '/smmix_webhook.log';
        error_log('[' . gmdate('Y-m-d H:i:s') . "][{$type}] {$message}\n", 3, $log_file);
    }

    private function normalize_language_code($lang)
    {
        $options = get_option($this->plugin_options);
        $mappings = [];

        if (!empty($options['language_code_mappings'])) {
            $lines = array_map('trim', explode("\n", $options['language_code_mappings']));
            foreach ($lines as $line) {
                if (empty($line)) continue;
                list($standard_code, $admin_code) = array_map('trim', explode('=', $line));
                $mappings[$standard_code] = $admin_code;
            }
        }

        // If there's a mapping for this language code, use it
        if (isset($mappings[$lang])) {
            return $mappings[$lang];
        }

        // If no mapping found, return original code
        return $lang;
    }

    private function detect_seo_plugin()
    {
        foreach ($this->seo_plugins as $plugin => $constant) {
            if (defined($constant)) return $plugin;
        }

        return false;
    }

    private function detect_language_plugin()
    {
        foreach ($this->language_plugins as $plugin => $constant) {
            if (defined($constant)) return $plugin;
        }

        return false;
    }

    public function activate_plugin()
    {
        $options = get_option($this->plugin_options);
        if (!$options || !isset($options['token'])) {
            if (!is_array($options)) $options = [];
            $options['token'] = substr(md5(uniqid(wp_rand(), true)), 0, 12);
            update_option($this->plugin_options, $options);
        }
        flush_rewrite_rules();
    }

    public function add_settings_link($links)
    {
        $settings_link = '<a href="options-general.php?page=' . $this->page_slug . '">Settings</a>';
        array_unshift($links, $settings_link);

        return $links;
    }

    public function register_webhook_endpoint()
    {
        global $wp_rewrite;

        add_rewrite_rule(
          '^smmix-webhook/?$',
          'index.php?smmix_webhook=1',
          'top'
        );

        add_filter('query_vars', function ($query_vars) {
            $query_vars[] = 'smmix_webhook';
            return $query_vars;
        });

        // Use template_include instead of template_redirect
        add_filter('template_include', function ($template) {
            global $wp_query;

            if (isset($wp_query->query_vars['smmix_webhook'])) {
                $this->handle_webhook();
                exit;
            }

            return $template;
        });

        $wp_rewrite->flush_rules(false);
    }

    public function handle_webhook()
    {
        global $wp_query;

        if (!isset($wp_query->query_vars['smmix_webhook'])) {
            return;
        }

        $options = get_option($this->plugin_options);
        if (empty($_GET['token']) || !isset($options['token']) || $_GET['token'] !== $options['token']) {
            $this->log_error('Invalid token', 'warning');
            wp_send_json([
              'posted' => false,
              'error' => 'Invalid token'
            ], 403);
            exit;
        }

        try {
            $json = file_get_contents('php://input');

            $data = json_decode($json, true);
            if ($data === null) {
                throw new Exception('Failed to parse JSON: ' . json_last_error_msg());
            }

            if (json_last_error() !== JSON_ERROR_NONE) {
                throw new Exception('Invalid JSON data: ' . json_last_error_msg());
            }

            $required_fields = ['article', 'alias', 'date'];
            foreach ($required_fields as $field) {
                if (empty($data[$field])) {
                    throw new Exception("Missing required field: {$field}");
                }
            }

            if (!strtotime($data['date'])) {
                throw new Exception('Invalid date format');
            }

            if (!is_array($data['article']) || empty($data['article'])) {
                throw new Exception('Article data must be an array');
            }

            foreach ($data['article'] as $lang => $content) {
                $required_content = ['title', 'text', 'meta_title', 'meta_description'];
                foreach ($required_content as $field) {
                    if (!isset($content[$field])) {
                        throw new Exception("Missing {$field} for language {$lang}");
                    }
                }
            }

            $created_posts = $this->process_incoming_data($data);

            wp_send_json([
              'posted' => true
            ], 200);
            exit;

        } catch (Exception $e) {
            $this->log_error($e->getMessage(), 'error');
            wp_send_json([
              'posted' => false,
              'error' => $e->getMessage()
            ], 500);
            exit;
        }
    }

    public function preserve_token($new_value, $old_value)
    {
        if (is_array($old_value) && isset($old_value['token'])) {
            $new_value['token'] = $old_value['token'];
        }

        return $new_value;
    }

    // Admin nav & settings
    public function add_admin_menu()
    {
        add_options_page(
          'SMMIX.tools Blog Settings',
          'SMMIX Blog',
          'manage_options',
          $this->page_slug,
          [$this, 'render_settings_page']
        );
    }

    public function register_settings()
    {
        register_setting($this->plugin_options, $this->plugin_options);

        add_settings_section(
          'smmix_blog_main',
          '',
          null,
          $this->page_slug
        );

        $lang_plugin = $this->detect_language_plugin();

        if ($lang_plugin) {
            add_settings_field(
              'language_code_mappings',
              'Language Code Mappings (Optional)',
              [$this, 'render_language_mappings_field'],
              $this->page_slug,
              'smmix_blog_main'
            );
        }

        add_settings_field(
          'blog_category_mappings',
          'Main Blog Category',
          [$this, 'render_blog_mappings_field'],
          $this->page_slug,
          'smmix_blog_main'
        );

        add_settings_field(
          'additional_category_mappings',
          'Additional Category Mappings (Optional)',
          [$this, 'render_additional_mappings_field'],
          $this->page_slug,
          'smmix_blog_main'
        );
    }

    public function render_settings_page()
    {
        if (!current_user_can('manage_options')) return;

        $options = get_option($this->plugin_options);
        if (!isset($options['token'])) {
            $options['token'] = substr(md5(uniqid(wp_rand(), true)), 0, 12);
            update_option($this->plugin_options, $options);
        }

        // Get current active tab
        $active_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : 'settings';
        ?>
			<div class="wrap">
				<h1>SMMIX.tools Blog Integration</h1>

				<h2 class="nav-tab-wrapper">
					<a href="?page=<?php echo esc_attr($this->page_slug); ?>&tab=settings"
						 class="nav-tab <?php echo $active_tab == 'settings' ? 'nav-tab-active' : ''; ?>">Settings</a>
					<a href="?page=<?php echo esc_attr($this->page_slug); ?>&tab=logs"
						 class="nav-tab <?php echo $active_tab == 'logs' ? 'nav-tab-active' : ''; ?>">Logs</a>
				</h2>

          <?php if ($active_tab == 'settings') : ?>
						<div style="background: white; padding: 20px; margin: 20px 0; border: 1px solid #ccc; border-radius: 5px;">
							<h2 style="margin-top: 0;">Webhook URL</h2>
							<code style="display: block; padding: 10px; background: #f5f5f5; margin: 10px 0;">
                  <?php echo esc_url(home_url($this->webhook_endpoint) . '?token=' . $options['token']) ?>
							</code>
							<p class="description">Use this URL in your SMMIX.tools dashboard to connect with your WordPress
								site.</p>
						</div>

						<div style="background: white; padding: 20px; margin: 20px 0; border: 1px solid #ccc; border-radius: 5px;">
							<h2 style="margin-top: 0;">Settings</h2>
							<form action="options.php" method="post">
                  <?php wp_nonce_field('smmix_blog_settings_action', 'smmix_blog_settings_nonce'); ?>
                  <?php
                  settings_fields($this->plugin_options);
                  do_settings_sections($this->page_slug);
                  submit_button('Save Settings');
                  ?>
							</form>
						</div>
          <?php elseif ($active_tab == 'logs') : ?>
              <?php $this->render_logs_tab(); ?>
          <?php endif; ?>
			</div>
        <?php
    }

    public function render_logs_tab()
    {
        $log_file = plugin_dir_path(__FILE__) . 'smmix_webhook.log';
        $log_content = '';

        if (file_exists($log_file)) {
            $log_content = file_get_contents($log_file);
        }

        // Handle log cleaning action
        if (isset($_POST['smmix_clean_logs']) && wp_verify_nonce($_POST['smmix_logs_nonce'], 'smmix_clean_logs_action')) {
            file_put_contents($log_file, '');
            $log_content = '';
            echo '<div class="notice notice-success is-dismissible"><p>' . esc_html__('Logs have been cleared successfully.', 'smmix-blog') . '</p></div>';
        }
        ?>
			<div style="background: white; padding: 20px; margin: 20px 0; border: 1px solid #ccc; border-radius: 5px;">
				<h2 style="margin-top: 0;">Webhook Logs</h2>
				<form method="post">
            <?php wp_nonce_field('smmix_clean_logs_action', 'smmix_logs_nonce'); ?>
					<textarea readonly
										style="width: 100%; height: 500px; font-family: monospace; margin-bottom: 10px;"><?php echo esc_textarea($log_content); ?></textarea>
					<input type="submit" name="smmix_clean_logs" class="button button-primary" value="Clean Logs">
				</form>
			</div>
        <?php
    }


    public function render_language_mappings_field()
    {
        $options = get_option($this->plugin_options);
        $value = isset($options['language_code_mappings']) ? $options['language_code_mappings'] : '';
        ?>
			<div class="mapping-field">
            <textarea name="<?php echo esc_html($this->plugin_options); ?>[language_code_mappings]" rows="5" cols="50"
											class="large-text code"><?php echo esc_textarea($value); ?></textarea>
				<p class="description">Enter language code mappings (one per line) in format: standard_code=admin_code</p>
				<p class="description">Example:</p>
				<pre>
uk=ua
cz=cs</pre>
				<p class="description">This defines how standard language codes (ISO 639-1) map to your admin panel language
					codes.</p>
			</div>
        <?php
    }

    public function render_blog_mappings_field()
    {
        $options = get_option($this->plugin_options);
        $value = isset($options['blog_category_mappings'])
          ? $options['blog_category_mappings'] : '';

        $lang_plugin = $this->detect_language_plugin();
        ?>
			<div class="mapping-field">
            <textarea name="<?php echo esc_html($this->plugin_options); ?>[blog_category_mappings]" rows="5" cols="50"
											class="large-text code"><?php echo esc_textarea($value); ?></textarea>
          <?php if ($lang_plugin): ?>
						<p class="description">Enter blog category mappings (one per line) in format: lang:slug</p>
						<p class="description">Example:</p>
						<pre>
uk:blog-uk
en:blog</pre>
						<p class="description">This defines the main blog category slug for each language.</p>
          <?php else: ?>
						<p class="description">Enter the blog category slug.</p>
						<p class="description">Example: blog</p>
          <?php endif; ?>
			</div>
        <?php
    }

    public function render_additional_mappings_field()
    {
        $options = get_option($this->plugin_options);
        $value = isset($options['additional_category_mappings']) ? $options['additional_category_mappings'] : '';

        $lang_plugin = $this->detect_language_plugin();
        ?>
			<div class="mapping-field">
            <textarea name="<?php echo esc_html($this->plugin_options); ?>[additional_category_mappings]" rows="10"
											cols="50" class="large-text code"><?php echo esc_textarea($value); ?></textarea>
          <?php if ($lang_plugin): ?>
						<p class="description">Enter category mappings in format:<br>key=<br>lang:slug<br>lang:slug</p>
						<p class="description">Example:</p>
						<pre>
news=
uk:novyny
en:news

support=
uk:pidtrymka
en:support</pre>
						<p class="description">The key (before =) should match the "rubric" value in the SMMIX.tools
							dashboard.</p>
          <?php else: ?>
						<p class="description">Enter category mappings in format:<br>key=slug</p>
						<p class="description">Example:</p>
						<pre>
news=novyny
support=pidtrymka</pre>
						<p class="description">The key (before =) should match the "rubric" value in the SMMIX.tools
							dashboard.</p>
          <?php endif; ?>
			</div>
        <?php
    }

    public function register_smmix_seoblog_params()
    {
        add_meta_box(
          'smmix_seoblog_params',
          'SMMIX Blog Parameters',
          [$this, 'show_smmix_seoblog_params'],
          'post',
          'normal',
          'default'
        );
    }

    public function show_smmix_seoblog_params($post)
    {
        $params = get_post_meta($post->ID, 'smmix_seoblog_params', true);
        if ($params) {
            $params_array = explode(';', $params);
            echo '<div class="smmix-params">';
            foreach ($params_array as $param) {
                if (empty($param)) continue;
                $parts = explode(':', $param);
                if (count($parts) !== 2) continue;
                $values = explode('|', $parts[1]);
                echo '<div class="param-row">';
                echo '<strong>' . esc_html(trim($parts[0])) . ':</strong> ' .
                     esc_html(implode(', ', array_map('trim', $values)));
                echo '</div>';
            }
            echo '</div>';
        } else {
            echo '<p>No SMMIX parameters found.</p>';
        }
    }

    private function parse_category_mappings(
      $blog_mappings,
      $additional_mappings
    )
    {
        $lang_plugin = $this->detect_language_plugin();

        $blog_slugs = [];
        $lines = array_map('trim', explode("\n", $blog_mappings));

        foreach ($lines as $line) {
            if (empty($line)) continue;

            if ($lang_plugin && strpos($line, ':') !== false) {
                // Language code is specified
                list($lang, $slug) = array_map('trim', explode(':', $line));
                $blog_slugs[$lang] = $slug;
            } else {
                // No language code specified, default slug
                $blog_slugs['default'] = $line;
            }
        }

        $category_slugs = [];
        $current_key = '';
        $lines = array_map('trim',
          explode("\n", $additional_mappings));

        foreach ($lines as $line) {
            if (empty($line)) continue;

            if (strpos($line, '=') !== false) {
                $current_key = trim(explode('=', $line)[0]);
                $category_slugs[$current_key] = [];
                $rest_of_line = trim(substr($line, strpos($line, '=') + 1));
                if ($rest_of_line !== '') {
                    if ($lang_plugin && strpos($rest_of_line, ':') !== false) {
                        list($lang, $slug) = array_map('trim', explode(':', $rest_of_line));
                        $category_slugs[$current_key][$lang] = $slug;
                    } else {
                        $category_slugs[$current_key]['default'] = $rest_of_line;
                    }
                }
            } else {
                if ($lang_plugin && strpos($line, ':') !== false) {
                    list($lang, $slug) = array_map('trim', explode(':', $line));
                    $category_slugs[$current_key][$lang] = $slug;
                } else {
                    $category_slugs[$current_key]['default'] = $line;
                }
            }
        }

        return [
          'blog_slugs' => $blog_slugs,
          'category_slugs' => $category_slugs
        ];
    }

    private function get_post_categories(
      $rubric_key = null,
      $current_lang = null
    )
    {
        $options = get_option($this->plugin_options);
        $mappings = $this->parse_category_mappings(
          $options['blog_category_mappings'] ?? '',
          $options['additional_category_mappings'] ?? ''
        );

        $categories = [];

        $lang_plugin = $this->detect_language_plugin();

        // Search for blog category for current language
        if ($lang_plugin && $current_lang && isset($mappings['blog_slugs'][$current_lang])) {
            $blog_slug = $mappings['blog_slugs'][$current_lang];
            $blog_category = get_term_by('slug', $blog_slug, 'category');

            if ($blog_category) {
                $categories[] = $blog_category->term_id;
            } else {
                $this->log_error("Blog category not found for lang {$current_lang}, slug: {$blog_slug}", 'warning');
            }
        } elseif (isset($mappings['blog_slugs']['default'])) {
            $blog_slug = $mappings['blog_slugs']['default'];
            $blog_category = get_term_by('slug', $blog_slug, 'category');

            if ($blog_category) {
                $categories[] = $blog_category->term_id;
            } else {
                $this->log_error("Blog category not found, slug: {$blog_slug}", 'warning');
            }
        }

        // Search for additional categories
        if ($rubric_key) {
            if ($lang_plugin && $current_lang && isset($mappings['category_slugs'][$rubric_key][$current_lang])) {
                $category_slug = $mappings['category_slugs'][$rubric_key][$current_lang];
                $additional_category = get_term_by('slug', $category_slug, 'category');

                if ($additional_category) {
                    $categories[] = $additional_category->term_id;
                } else {
                    $this->log_error("Additional category not found for key {$rubric_key}, lang {$current_lang}, slug: {$category_slug}", 'warning');
                }
            } elseif (isset($mappings['category_slugs'][$rubric_key]['default'])) {
                $category_slug = $mappings['category_slugs'][$rubric_key]['default'];
                $additional_category = get_term_by('slug', $category_slug, 'category');

                if ($additional_category) {
                    $categories[] = $additional_category->term_id;
                } else {
                    $this->log_error("Additional category not found for key {$rubric_key}, slug: {$category_slug}", 'warning');
                }
            }
        }

        return array_unique($categories);
    }

    private function process_incoming_data($data)
    {
        $lang_plugin = $this->detect_language_plugin();
        $seo_plugin = $this->detect_seo_plugin();

        $created_posts = [];

        if ($lang_plugin === 'wp_multilang') {
            // WP Multilang
            $post_title = '';
            $post_content = '';
            $meta_title = '';
            $meta_description = '';

            foreach ($data['article'] as $lang => $article) {
                $normalized_lang = $this->normalize_language_code($lang);

                $post_title .= '[:' . $normalized_lang . ']' . $article['title'];
                $post_content .= '[:' . $normalized_lang . ']' . $article['text'];
                if ($seo_plugin) {
                    $meta_title .= '[:' . $normalized_lang . ']' . $article['meta_title'];
                    $meta_description .= '[:' . $normalized_lang . ']' . $article['meta_description'];
                }
            }

            $post_title .= '[:]';
            $post_content .= '[:]';
            if ($seo_plugin) {
                $meta_title .= '[:]';
                $meta_description .= '[:]';
            }

            $categories = $this->get_post_categories($data['rubric']);

            $post_id = wp_insert_post([
              'post_type' => 'post',
              'post_status' => 'publish',
              'post_title' => $post_title,
              'post_content' => $post_content,
              'post_name' => $data['alias'],
              'post_date' => $data['date']
            ]);

            if (is_wp_error($post_id)) {
                throw new Exception('Failed to create post: ' . esc_html($post_id->get_error_message()));
            }

            if (!empty($categories)) {
                wp_set_post_categories($post_id, $categories, true);
            }

            if ($seo_plugin) {
                $this->handle_seo_meta($post_id, [
                  'title' => $meta_title,
                  'description' => $meta_description
                ], $seo_plugin);
            }

            $image_data = $this->handle_image_upload($data['image'], $post_id, $data['params']);
            if ($image_data) {
                set_post_thumbnail($post_id, $image_data['id']);
                if ($seo_plugin) {
                    $this->handle_seo_meta($post_id, ['title' => $meta_title, 'description' => $meta_description], $seo_plugin, $image_data);
                }
            }

            $created_posts[] = $post_id;

        } elseif ($lang_plugin === 'polylang' || $lang_plugin === 'wpml') {
            // Polylang / WPML
            $translations = [];
            $first_post_id = null;

            foreach ($data['article'] as $lang => $article) {
                $normalized_lang = $this->normalize_language_code($lang);
                $categories = $this->get_post_categories($data['rubric'], $normalized_lang);

                $post_id = wp_insert_post([
                  'post_type' => 'post',
                  'post_status' => 'publish',
                  'post_title' => $article['title'],
                  'post_content' => $article['text'],
                  'post_name' => $data['alias'] . ($first_post_id ? '-' . $normalized_lang : ''),
                  'post_date' => $data['date']
                ]);

                if (is_wp_error($post_id)) {
                    $this->log_error("Failed to create post for language {$normalized_lang}: " . $post_id->get_error_message(), 'error');
                    continue;
                }

                wp_set_post_categories($post_id, $categories, false);

                $created_posts[] = $post_id;

                if (!$first_post_id) $first_post_id = $post_id;

                if ($lang_plugin === 'polylang') {
                    pll_set_post_language($post_id, $normalized_lang);
                    $translations[$normalized_lang] = $post_id;
                } else { // WPML
                    if ($first_post_id === $post_id) {
                        do_action('wpml_set_element_language_details', [
                          'element_id' => $post_id,
                          'element_type' => 'post_post',
                          'language_code' => $normalized_lang,
                          'source_language_code' => null
                        ]);
                    } else {
                        do_action('wpml_set_element_language_details', [
                          'element_id' => $post_id,
                          'element_type' => 'post_post',
                          'language_code' => $normalized_lang,
                          'source_language_code' => $this->normalize_language_code(array_key_first($data['article'])),
                          'trid' => $first_post_id
                        ]);
                    }
                }

                // Meta tags
                if ($seo_plugin) {
                    $this->handle_seo_meta($post_id, [
                      'title' => $article['meta_title'],
                      'description' => $article['meta_description']
                    ], $seo_plugin);
                }
            }

            if ($lang_plugin === 'polylang' && !empty($translations)) {
                pll_save_post_translations($translations);
            }
        } else {
            // Without multilang plugin
            $default_lang = array_key_first($data['article']);
            $article = $data['article'][$default_lang];
            $categories = $this->get_post_categories($data['rubric']);

            $post_id = wp_insert_post([
              'post_type' => 'post',
              'post_status' => 'publish',
              'post_title' => $article['title'],
              'post_content' => $article['text'],
              'post_name' => $data['alias'],
              'post_date' => $data['date']
            ]);

            if (is_wp_error($post_id)) {
                throw new Exception('Failed to create post: ' . esc_html($post_id->get_error_message()));
            }

            if (!empty($categories)) {
                wp_set_post_categories($post_id, $categories, false);
            }

            $created_posts[] = $post_id;

            if ($seo_plugin) {
                $this->handle_seo_meta($post_id, [
                  'title' => $article['meta_title'],
                  'description' => $article['meta_description']
                ], $seo_plugin);
            }
        }

        // Add image
        $image_data = null;
        if (!empty($created_posts)) {
            $image_data = $this->handle_image_upload($data['image'], $created_posts[0], $data['params']);
            if ($image_data) {
                foreach ($created_posts as $post_id) {
                    set_post_thumbnail($post_id, $image_data['id']);
                    // Meta data for image
                    if ($seo_plugin) {
                        $current_meta = [];

                        switch ($seo_plugin) {
                            case 'yoast':
                                $current_meta['title'] = get_post_meta($post_id, '_yoast_wpseo_title', true);
                                $current_meta['description'] = get_post_meta($post_id, '_yoast_wpseo_metadesc', true);
                                break;
                            case 'rank_math':
                                $current_meta['title'] = get_post_meta($post_id, 'rank_math_title', true);
                                $current_meta['description'] = get_post_meta($post_id, 'rank_math_description', true);
                                break;
                            case 'all_in_one':
                                $current_meta['title'] = get_post_meta($post_id, '_aioseop_title', true);
                                $current_meta['description'] = get_post_meta($post_id, '_aioseop_description', true);
                                break;
                        }

                        $this->handle_seo_meta($post_id, $current_meta,
                          $seo_plugin, $image_data);
                    }
                }
            } else {
                $this->log_error("Failed to upload image from URL: {$data['image']}", 'warning');
            }
        }

        // Add parameters
        if (!empty($data['params'])) {
            foreach ($created_posts as $post_id) {
                update_post_meta($post_id, 'smmix_seoblog_params', $data['params']);
            }
        }

        return $created_posts;
    }

    private function handle_image_upload($image_source, $post_id, $params = '')
    {
        $upload_dir = wp_upload_dir();

        // If URL is provided and valid, download image from URL
        if (!empty($image_source) && filter_var($image_source, FILTER_VALIDATE_URL)) {
            $image_data = @file_get_contents($image_source);
            if ($image_data === false) {
                $this->log_error("Failed to download image: {$image_source}", 'error');
                return null;
            }
            $filename = basename($image_source);
            $file = $upload_dir['path'] . '/' . $filename;

            if (@file_put_contents($file, $image_data)) {
                return $this->create_attachment($file, $filename, $post_id, $upload_dir);
            }
            return null;
        }

        // If no URL provided, use local directory for images
        $directory = ABSPATH . 'smmix_blog_photo';
        $this->log_error('Directory used for image search: ' . $directory, 'info');

        // Parse parameters for potential folder matching
        $pattern = '/[^a-zA-Z0-9_\-]+/u';
        $param_keys = preg_split($pattern, $params, -1, PREG_SPLIT_NO_EMPTY);

        // First try to find images in the main directory
        $images = $this->get_images_in_directory($directory);

        // If no images in main directory, try to find in matching subfolders
        if (empty($images)) {
            $subfolders = glob($directory . '/*', GLOB_ONLYDIR);
            $matching_folders = [];

            // Find matching folders based on params
            if (!empty($subfolders) && !empty($param_keys)) {
                foreach ($subfolders as $subfolder) {
                    $folder_name = strtolower(basename($subfolder));
                    foreach ($param_keys as $key) {
                        $key_lower = strtolower($key);
                        if (strpos($folder_name, $key_lower) !== false) {
                            $matching_folders[] = $subfolder;
                            $this->log_error("Matched folder '{$folder_name}' for key '{$key}'", 'info');
                            break;
                        }
                    }
                }
            }

            // If we found matching folders, select one randomly and get images
            if (!empty($matching_folders)) {
                $selected_folder = $matching_folders[array_rand($matching_folders)];
                $this->log_error('Selected folder: ' . $selected_folder, 'info');
                $images = $this->get_images_in_directory($selected_folder);

                if (empty($images)) {
                    $this->log_error("No images found in matching folder: {$selected_folder}", 'warning');
                    return null;
                }
            } else {
                $this->log_error('No images found in directory and no matching subfolder based on params', 'warning');
                return null;
            }
        }

        // Select a random image from the found images
        $selected_image = $images[array_rand($images)];

        // Upload the selected image
        $image_data = @file_get_contents($selected_image);
        if ($image_data === false) {
            $this->log_error("Failed to read image: {$selected_image}", 'error');
            return null;
        }

        $filename = basename($selected_image);
        $file = $upload_dir['path'] . '/' . $filename;

        if (@file_put_contents($file, $image_data)) {
            return $this->create_attachment($file, $filename, $post_id, $upload_dir);
        }

        return null;
    }

    private function create_attachment($file, $filename, $post_id, $upload_dir)
    {
        require_once(ABSPATH . 'wp-admin/includes/image.php');

        $attachment = [
          'post_mime_type' => wp_check_filetype($filename, null)['type'],
          'post_title' => sanitize_file_name($filename),
          'post_content' => '',
          'post_status' => 'inherit'
        ];

        $attach_id = wp_insert_attachment($attachment, $file, $post_id);
        $this->log_error('Attachment created with ID: ' . print_r($attach_id, true), 'info');

        if (!is_wp_error($attach_id)) {
            $attach_data = wp_generate_attachment_metadata($attach_id, $file);
            wp_update_attachment_metadata($attach_id, $attach_data);
            return [
              'id' => $attach_id,
              'url' => $upload_dir['url'] . '/' . $filename
            ];
        }

        $this->log_error('Failed to create attachment: ' . $attach_id->get_error_message(), 'error');
        return null;
    }


    private function get_images_in_directory($directory)
    {
        $directory = rtrim($directory, '/');

        // Get all files in the directory
        $all_files = glob("$directory/*.*");
        if (empty($all_files)) {
            return [];
        }

        // Define pattern of image extensions to check
        $image_pattern = '/\.(jpe?g|png|gif|svg|avif|webp)$/i';

        // Filter files to only include images
        $images = array_filter($all_files, function ($file) use ($image_pattern) {
            return preg_match($image_pattern, $file);
        });

        return array_values($images); // Reset array keys
    }

    private function handle_seo_meta($post_id, $meta_data, $seo_plugin, $image_data = null)
    {
        if (!$seo_plugin || empty($meta_data)) {
            return;
        }

        $lang_plugin = $this->detect_language_plugin();

        switch ($seo_plugin) {
            case 'yoast':
                if ($lang_plugin === 'wp_multilang') {
                    if (!empty($meta_data['title'])) {
                        update_post_meta($post_id, '_yoast_wpseo_title', $meta_data['title']);
                        update_post_meta($post_id, '_yoast_wpseo_opengraph-title', $meta_data['title']);
                        update_post_meta($post_id, '_yoast_wpseo_twitter-title', $meta_data['title']);
                    }
                    if (!empty($meta_data['description'])) {
                        update_post_meta($post_id, '_yoast_wpseo_metadesc', $meta_data['description']);
                        update_post_meta($post_id, '_yoast_wpseo_opengraph-description', $meta_data['description']);
                        update_post_meta($post_id, '_yoast_wpseo_twitter-description', $meta_data['description']);
                    }
                } else {
                    if (!empty($meta_data['title'])) {
                        $title = preg_replace('/\[:[a-z]{2}\]|\[:\]/', '', $meta_data['title']);
                        update_post_meta($post_id, '_yoast_wpseo_title', $title);
                        update_post_meta($post_id, '_yoast_wpseo_opengraph-title', $title);
                        update_post_meta($post_id, '_yoast_wpseo_twitter-title', $title);
                    }
                    if (!empty($meta_data['description'])) {
                        $description = preg_replace('/\[:[a-z]{2}\]|\[:\]/', '', $meta_data['description']);
                        update_post_meta($post_id, '_yoast_wpseo_metadesc', $description);
                        update_post_meta($post_id, '_yoast_wpseo_opengraph-description', $description);
                        update_post_meta($post_id, '_yoast_wpseo_twitter-description', $description);
                    }
                }
                if ($image_data) {
                    update_post_meta($post_id, '_yoast_wpseo_opengraph-image', $image_data['url']);
                    update_post_meta($post_id, '_yoast_wpseo_opengraph-image-id', $image_data['id']);
                    update_post_meta($post_id, '_yoast_wpseo_twitter-image', $image_data['url']);
                    update_post_meta($post_id, '_yoast_wpseo_twitter-image-id', $image_data['id']);
                }
                break;

            case 'rank_math':
                if ($lang_plugin === 'wp_multilang') {
                    if (!empty($meta_data['title'])) {
                        update_post_meta($post_id, 'rank_math_title', $meta_data['title']);
                        update_post_meta($post_id, 'rank_math_facebook_title', $meta_data['title']);
                        update_post_meta($post_id, 'rank_math_twitter_title', $meta_data['title']);
                    }
                    if (!empty($meta_data['description'])) {
                        update_post_meta($post_id, 'rank_math_description', $meta_data['description']);
                        update_post_meta($post_id, 'rank_math_facebook_description', $meta_data['description']);
                        update_post_meta($post_id, 'rank_math_twitter_description', $meta_data['description']);
                    }
                } else {
                    if (!empty($meta_data['title'])) {
                        $title = preg_replace('/\[:[a-z]{2}\]|\[:\]/', '', $meta_data['title']);
                        update_post_meta($post_id, 'rank_math_title', $title);
                        update_post_meta($post_id, 'rank_math_facebook_title', $title);
                        update_post_meta($post_id, 'rank_math_twitter_title', $title);
                    }
                    if (!empty($meta_data['description'])) {
                        $description = preg_replace('/\[:[a-z]{2}\]|\[:\]/', '', $meta_data['description']);
                        update_post_meta($post_id, 'rank_math_description', $description);
                        update_post_meta($post_id, 'rank_math_facebook_description', $description);
                        update_post_meta($post_id, 'rank_math_twitter_description', $description);
                    }
                }
                if ($image_data) {
                    update_post_meta($post_id, 'rank_math_facebook_image', $image_data['url']);
                    update_post_meta($post_id, 'rank_math_facebook_image_id', $image_data['id']);
                    update_post_meta($post_id, 'rank_math_twitter_image', $image_data['url']);
                    update_post_meta($post_id, 'rank_math_twitter_image_id', $image_data['id']);
                }
                break;

            case 'all_in_one':
                if ($lang_plugin === 'wp_multilang') {
                    if (!empty($meta_data['title'])) {
                        update_post_meta($post_id, '_aioseop_title', $meta_data['title']);
                    }
                    if (!empty($meta_data['description'])) {
                        update_post_meta($post_id, '_aioseop_description', $meta_data['description']);
                    }
                } else {
                    if (!empty($meta_data['title'])) {
                        $title = preg_replace('/\[:[a-z]{2}\]|\[:\]/', '', $meta_data['title']);
                        update_post_meta($post_id, '_aioseop_title', $title);
                    }
                    if (!empty($meta_data['description'])) {
                        $description = preg_replace('/\[:[a-z]{2}\]|\[:\]/', '', $meta_data['description']);
                        update_post_meta($post_id, '_aioseop_description', $description);
                    }
                }
                if ($image_data) {
                    $og_settings = get_post_meta($post_id, '_aioseop_opengraph_settings', true) ?: [];
                    $og_settings = array_merge($og_settings, [
                      'aioseop_opengraph_settings_title' => $meta_data['title'] ?? '',
                      'aioseop_opengraph_settings_desc' => $meta_data['description'] ?? '',
                      'aioseop_opengraph_settings_customimg' => $image_data['url'],
                      'aioseop_opengraph_settings_image' => $image_data['url']
                    ]);
                    update_post_meta($post_id, '_aioseop_opengraph_settings', $og_settings);
                }
                break;
        }
    }

}

// Init
function SMMIX_Blog_Integration_Init()
{
    new SMMIX_Blog_Integration();
}

add_action('plugins_loaded', 'SMMIX_Blog_Integration_Init');